<?php

namespace App\Models;

use Illuminate\Contracts\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Answer extends Model
{
    use HasFactory, SoftDeletes;

    protected $fillable = [
        'user_id',
        'question_id ',
        'answer',
        'video_path',
        'image_path',
        'file_path',
    ];

    public function scopeLatest(Builder $builder)
    {
        return $builder->orderBy(static::CREATED_AT, 'desc');
    }

    public function comments()
    {
        return $this->morphMany(Comment::class, 'comment')->where('confirm', true)->latest();
    }

    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function question()
    {
        return $this->belongsTo(Question::class);
    }

    public function likes()
    {
        return $this->morphToMany(User::class, 'like')->where('action', 'like')->withTimestamps();
    }

    public function disLikes()
    {
        return $this->morphToMany(User::class, 'like')->where('action', 'dislike')->withTimestamps();
    }

}
