<?php

namespace App\Repository\Eloquent;

use App\Http\Resources\CategoryResource;
use App\Http\Resources\CommentResource;
use App\Http\Resources\ItemResource;
use App\Models\Answer;
use App\Models\Category;
use App\Models\Item;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class CommentRepository extends BaseRepository{

    public function getComments(Request $request)
    {

        if($request->type == 'item')
            $comments = Item::find($request->id)->comments;
        elseif($request->type == 'answer')
            $comments = Answer::find($request->id)->comments;

        $comments->each(function ($comment)
        {

            $comment['likes_count'] = $comment->likes->count();
            $comment['dislikes_count'] = $comment->disLikes->count();

            $comment['posted_at'] = verta($comment['created_at'])->formatDifference();
            $comment['who_posted'] = $comment->user->first_name;
            

            $comment['action'] = null;
            $user = auth('sanctum')->user();
            if(!is_null($user)){
                if(!is_null($comment->likes)){
                    $like =  $comment->likes->search(function($like) use($user){
                            return $like->id == $user->id;
                        });
                    if($like !== false){
                        $comment['action'] = 'like';
                    }
                }

                if(!is_null($comment->disLikes)){

                    $disLike =  $comment->disLikes->search(function($like) use($user){
                        return $like->id == $user->id;
                    });
                    if($disLike !== false){
                        $comment['action'] = 'dislike';
                    }
                }
            }
            
        });

        return $this->sendResponse(CommentResource::collection($comments));
    }

    public function postComment(Request $request)
    {

        $type = null;
        if($request->type === 'item')
            $type = Item::find($request->id);
        else
            $type = Answer::find($request->id);

        $type->comments()->create([
            'content' => $request->content,
            'user_id' => Auth::user()->id,
            'confirm' => false,
        ]);
        
        return $this->sendResponse('نظر شما پس از تایید نمایش داده خواهد شد');
    }
    
    

}

?>